#ifndef PYTHONIC_IO__IO_TEXTIOWRAPPER_SEEK_HPP
#define PYTHONIC_IO__IO_TEXTIOWRAPPER_SEEK_HPP

#include "pythonic/include/io/_io/TextIOWrapper/seek.hpp"
#include "pythonic/builtins/file/seek.hpp"

#endif
