#ifndef PYTHONIC_INCLUDE_NUMPY_FMIN_HPP
#define PYTHONIC_INCLUDE_NUMPY_FMIN_HPP

#include "pythonic/include/numpy/minimum.hpp"

PYTHONIC_NS_BEGIN

namespace numpy
{
  USING_FUNCTOR(fmin, minimum);
}
PYTHONIC_NS_END

#endif
