#ifndef PYTHONIC_INCLUDE_NUMPY_DIAGFLAT_HPP
#define PYTHONIC_INCLUDE_NUMPY_DIAGFLAT_HPP

#include "pythonic/include/numpy/diag.hpp"

PYTHONIC_NS_BEGIN

namespace numpy
{
  USING_FUNCTOR(diagflat, diag);
}
PYTHONIC_NS_END

#endif
