#ifndef PYTHONIC_MATH_SQRT_HPP
#define PYTHONIC_MATH_SQRT_HPP

#include "pythonic/include/math/sqrt.hpp"

#include "pythonic/utils/functor.hpp"
#include <cmath>

PYTHONIC_NS_BEGIN

namespace math
{
}
PYTHONIC_NS_END

#endif
